@@ -4,15 +4,22 @@
   support <- unlist(magick_config()[testfor])
   has <- paste(names(which(support)), collapse = ", ")
   not <- paste(names(which(!support)), collapse = ", ")
-
+  threads <- magick_threads(0)
   packageStartupMessage(sprintf("Linking to ImageMagick %s\nEnabled features: %s\nDisabled features: %s",
                                 as.character(magick_config()$version), has, not))
+  if(isTRUE(threads > 1))
+    packageStartupMessage(sprintf("Using %d threads", threads))
 
   # For RStudio
   autoviewer_enable()
 }
 
 .onLoad <- function(lib, pkg){
+  # Try to please cran...
+  if(is_check() && isTRUE(magick_threads() > 2)){
+    magick_threads(2)
+  }
+
   # Set tempdir to R session
   set_magick_tempdir(tempdir())
 
@@ -36,10 +43,6 @@
     } else if(file.exists(fontdir)){
       Sys.setenv(FONTCONFIG_PATH = fontdir)
     }
-  } else if(is_mac()){
-    # Workaround for R's built-in OpenMP conflicts
-    # https://github.com/ropensci/magick/issues/170
-    Sys.setenv(KMP_DUPLICATE_LIB_OK = 'TRUE')
   }
   register_s3_method("knitr", "knit_print", "magick-image")
 }
@@ -72,3 +75,6 @@ is_mac <- function(){
   grepl("darwin", R.Version()$platform)
 }
 
+is_check <- function(){
+  grepl('magick.Rcheck', getwd(), fixed = TRUE)
+}
