@@ -1,4 +1,4 @@
-/* Jeroen Ooms (2017)
+/* Jeroen Ooms (2018)
  * Bindings to vectorized image manipulations.
  * See API: https://www.imagemagick.org/Magick++/STL.html
  */
@@ -60,14 +60,23 @@ XPtrImage magick_image_readbitmap_double(Rcpp::NumericVector x){
 }
 
 // [[Rcpp::export]]
-XPtrImage magick_image_readbin(Rcpp::RawVector x, Rcpp::CharacterVector density, Rcpp::IntegerVector depth, bool strip = false){
+XPtrImage magick_image_readbin(Rcpp::RawVector x, Rcpp::CharacterVector density, Rcpp::IntegerVector depth,
+                               bool strip, Rcpp::CharacterVector defines){
   XPtrImage image = create();
 #if MagickLibVersion >= 0x689
   Magick::ReadOptions opts = Magick::ReadOptions();
+#if MagickLibVersion >= 0x690
+  opts.quiet(1);
+#endif
   if(density.size())
     opts.density(std::string(density.at(0)).c_str());
   if(depth.size())
     opts.depth(depth.at(0));
+  if(defines.size()){
+    Rcpp::CharacterVector names = defines.names();
+    for(int i = 0; i < defines.size(); i++)
+      MagickCore::SetImageOption(opts.imageInfo(), names.at(i), defines.at(i));
+  }
   Magick::readImages(image.get(), Magick::Blob(x.begin(), x.length()), opts);
 #else
   Magick::readImages(image.get(), Magick::Blob(x.begin(), x.length()));
@@ -78,14 +87,23 @@ XPtrImage magick_image_readbin(Rcpp::RawVector x, Rcpp::CharacterVector density,
 }
 
 // [[Rcpp::export]]
-XPtrImage magick_image_readpath(Rcpp::CharacterVector paths, Rcpp::CharacterVector density, Rcpp::IntegerVector depth, bool strip = false){
+XPtrImage magick_image_readpath(Rcpp::CharacterVector paths, Rcpp::CharacterVector density, Rcpp::IntegerVector depth,
+                                bool strip, Rcpp::CharacterVector defines){
   XPtrImage image = create();
 #if MagickLibVersion >= 0x689
   Magick::ReadOptions opts = Magick::ReadOptions();
+#if MagickLibVersion >= 0x690
+  opts.quiet(1);
+#endif
   if(density.size())
     opts.density(std::string(density.at(0)).c_str());
   if(depth.size())
     opts.depth(depth.at(0));
+  if(defines.size()){
+    Rcpp::CharacterVector names = defines.names();
+    for(int i = 0; i < defines.size(); i++)
+      MagickCore::SetImageOption(opts.imageInfo(), names.at(i), defines.at(i));
+  }
   for(int i = 0; i < paths.size(); i++)
     Magick::readImages(image.get(), std::string(paths[i]), opts);
 #else
@@ -160,6 +178,23 @@ Rcpp::RawVector magick_image_write_frame(XPtrImage input, const char * format, s
   return res;
 }
 
+// [[Rcpp::export]]
+Rcpp::IntegerVector magick_image_write_integer(XPtrImage input){
+  if(input->size() != 1)
+    throw std::runtime_error("Image must have single frame to write a native raster");
+  Frame frame = input->front();
+  Magick::Geometry size(frame.size());
+  size_t width = size.width();
+  size_t height = size.height();
+  Magick::Blob output;
+  frame.write(&output, "RGBA", 8L);
+  Rcpp::IntegerVector res(output.length() / 4);
+  memcpy(res.begin(), output.data(), output.length());
+  res.attr("class") = Rcpp::CharacterVector::create("nativeRaster");
+  res.attr("dim") = Rcpp::NumericVector::create(height, width);
+  return res;
+}
+
 // [[Rcpp::export]]
 XPtrImage magick_image_display( XPtrImage image, bool animate){
 #ifndef MAGICKCORE_X11_DELEGATE
@@ -198,3 +233,30 @@ XPtrImage magick_image_strip( XPtrImage input){
   for_each (output->begin(), output->end(), Magick::stripImage());
   return output;
 }
+
+// [[Rcpp::export]]
+XPtrImage magick_image_separate( XPtrImage input, const char * channel){
+  XPtrImage output = create();
+#if MagickLibVersion >= 0x687
+  separateImages( output.get(), input->front(), Channel(channel));
+#else
+  throw std::runtime_error("Your imagemagick is too old for separateImages");
+#endif
+  return output;
+}
+
+// [[Rcpp::export]]
+XPtrImage magick_image_combine( XPtrImage input, const char * colorspace, const char * channel){
+  Frame x;
+#if MagickLibVersion >= 0x700
+  combineImages(&x, input->begin(), input->end(), Channel(channel), ColorSpace(colorspace));
+#elif MagickLibVersion >= 0x687
+  combineImages(&x, input->begin(), input->end(), Channel(channel));
+  x.colorspaceType(ColorSpace(colorspace));
+#else
+  throw std::runtime_error("Your imagemagick is too old for combineImages");
+#endif
+  XPtrImage output = create(1);
+  output->push_back(x);
+  return output;
+}
